package cubo3d;

import java.awt.Color;
import javax.swing.JPanel;

public class MatUtil {

    public Color faceCor( Face3D face, double[][] fontesLuz, Color cor ) {
        double[] v1 = face.getVertices().get( 0 ).getVetorCopia();
        double[] v2 = face.getVertices().get( 1 ).getVetorCopia();
        double[] v3 = face.getVertices().get( 2 ).getVetorCopia();
        
        double[] vmedio = { 
            ( v1[0]+v2[0]+v3[0] ) / 3.0d,
            ( v1[1]+v2[1]+v3[1] ) / 3.0d,
            ( v1[2]+v2[2]+v3[2] ) / 3.0d,            
        };
        
        double[] vn = vetorNormal( v1 , v2, v3 );
        double[] v_centro = mult( vmedio, -1 );
        
        double a = angulo( vn, v_centro );
        if ( a < ( Math.PI / 2.0d ) )
            vn = mult( vn, -1 );
                
        double quantCor = 0;
        for( double[] fonteLuz : fontesLuz ) {
            double[] vf_luz = sub( fonteLuz, vmedio );
            double a_luz = angulo( vn, vf_luz );
            if ( a_luz < ( Math.PI / 2.0d ) )
                quantCor +=( ( Math.PI / 2.0d ) - a_luz ) / ( Math.PI / 2.0d );
        }                
                
        int r = (int)( quantCor * cor.getRed() );
        int g = (int)( quantCor * cor.getGreen() );
        int b = (int)( quantCor * cor.getBlue() );        

        if ( r > 255 ) r = 255;
        if ( g > 255 ) g = 255;
        if ( b > 255 ) b = 255;

        return new Color( r, g, b );                
    }
    
    public double[] vetorNormal( Face3D face ) {
        double[] v1 = face.getVertices().get( 0 ).getVetorCopia();
        double[] v2 = face.getVertices().get( 1 ).getVetorCopia();
        double[] v3 = face.getVertices().get( 2 ).getVetorCopia();
        
        double[] vmedio = { 
            ( v1[0]+v2[0]+v3[0] ) / 3.0d,
            ( v1[1]+v2[1]+v3[1] ) / 3.0d,
            ( v1[2]+v2[2]+v3[2] ) / 3.0d,            
        };
        
        double[] vn = vetorNormal( v1 , v2, v3 );        
        double[] v_centro = mult( vmedio, -1 );
        
        double a = angulo( vn, v_centro );
        if ( a < ( Math.PI / 2.0d ) )
            vn = mult( vn, -1 );
        
        return vn;
    }
                        
    public double[] vetorNormal( double[] p0, double[] p1, double[] p2 ) {
        double x1 = p1[0]-p0[0];
        double y1 = p1[1]-p0[1];
        double z1 = p1[2]-p0[2];
        
        double x2 = p2[0]-p0[0];
        double y2 = p2[1]-p0[1];
        double z2 = p2[2]-p0[2];
        
        return new double[] {
            y1*z2 - y2*z1, 
            x2*z1 - x1*z2,
            x1*y2 - x2*y1
        };                
    }
    
    public double[] setComprimentoVetorNormal( double[] vn, double c ) {        
        double r = Math.sqrt( vn[0]*vn[0] + vn[1]*vn[1] + vn[2]*vn[2] );
        return new double[] { 
            vn[0] * c / r,
            vn[1] * c / r,
            vn[2] * c / r
        };
    }

    public double angulo( double[] u, double[] v ) {
        double c_u = Math.sqrt( u[0]*u[0] + u[1]*u[1] + u[2]*u[2] );
        double c_v = Math.sqrt( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );        
        double uv_prod = ( u[0]*v[0] + u[1]*v[1] + u[2]*v[2] );
                
        return Math.acos( uv_prod / ( c_u * c_v ) );        
    }
    
    public int[] projecao2d( double[] v, JPanel p ) {
        int xc = p.getWidth()  / 2;
        int yc = p.getHeight() / 2;
            
        int x = xc + (int)v[0];
        int y = yc - (int)v[1];
        
        return new int[] { x, y };
    }
            
    public double[] soma( double[] u, double[] v ) {
        return new double[] { u[0]+v[0],  u[1]+v[1], u[2]+v[2] };
    }
    
    public double[] sub( double[] u, double[] v ) {
        return new double[] { u[0]-v[0],  u[1]-v[1], u[2]-v[2] };
    }
    
    public double[] mult( double[] u, double[] v ) {
        return new double[] { u[0]*v[0],  u[1]*v[1], u[2]*v[2] };        
    }
    
    
    public double[] mult( double[] u, double escalar ) {
        return new double[] { u[0]*escalar,  u[1]*escalar, u[2]*escalar };        
    }
    
}
