package cubo3d;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class Principal extends JPanel implements MouseListener, MouseMotionListener, KeyListener {
    
    private final double D = 1000;
    
    private Forma3D forma;
    private int mouseX = -1;
    private int mouseY = -1;
    private final Transformador transformador = new Transformador();
    private final MatUtil math = new MatUtil();
    
    private double[][] fontesLuz = {
        { 100, 300, 1000 },
        { -1000, 500, 500 },
        { 1000, 500, 0 },
        { 500, -1000, 0 },
    };
    
    private boolean pintarVetorNormal = false;
    private double comprimentoVetorNormal = 50.0d;
    private Color corVetorNormal = Color.YELLOW;
    
    private boolean pintarArestas = false;
    private boolean pintarFaces = true;
    
    private char rotCH = 'w';
    
    public Principal() {
        super.setBackground( Color.BLACK );
        super.addMouseListener( this ); 
        super.addMouseMotionListener( this ); 
        super.addKeyListener( this );
        super.setFocusable( true );                 
    }
    
    @Override
    public void paintComponent( Graphics g ) {
        super.paintComponent( g );
        
        if ( forma != null ) {                        
            forma.getVertices().forEach( (v) -> { v.carregaCopia(); } ); 
            transformador.perspectiva( forma, D );
            
            Color arestasCor = forma.getArestasCor();
            Color facesCor = forma.getFacesCor();
                        
            for( Face3D f : forma.getFaces() ) {
                double[] vn = math.vetorNormal( f );
                if ( vn[2] < 0 )
                    continue;
                
                if ( pintarFaces ) {
                    int nvertices = f.getVertices().size();
                    int[] x_lista = new int[ nvertices ];
                    int[] y_lista = new int[ nvertices ];
                    for( int i = 0; i < nvertices; i++ ) {
                        double[] v = f.getVertices().get( i ).getVetorCopia();                                

                        int[] xy = math.projecao2d( v, this );
                        x_lista[i] = xy[0];
                        y_lista[i] = xy[1];
                    }

                    Color cor = math.faceCor( f, fontesLuz, facesCor );

                    g.setColor( cor ); 
                    g.fillPolygon( x_lista, y_lista, nvertices );
                } 
                
                if ( pintarVetorNormal ) {
                    double[] v1 = f.getVertices().get( 0 ).getVetorCopia();
                    double[] v2 = f.getVertices().get( 1 ).getVetorCopia();
                    double[] v3 = f.getVertices().get( 2 ).getVetorCopia();

                    double[] vmedio = { 
                        ( v1[0]+v2[0]+v3[0] ) / 3.0d,
                        ( v1[1]+v2[1]+v3[1] ) / 3.0d,
                        ( v1[2]+v2[2]+v3[2] ) / 3.0d,            
                    };
                    
                    double[] c_vn = math.setComprimentoVetorNormal( vn, comprimentoVetorNormal );
                    double[] vvn = math.soma( c_vn, vmedio );
                    
                    int[] lp1 = math.projecao2d( vmedio, this );
                    int[] lp2 = math.projecao2d( vvn, this );
                    
                    g.setColor( corVetorNormal );
                    g.drawLine( lp1[0], lp1[1], lp2[0], lp2[1] );
                }
            }
            
            if ( pintarArestas ) {
                for( Aresta3D a : forma.getArestas() ) {
                    double[] v1 = a.getV1().getVetorCopia();
                    double[] v2 = a.getV2().getVetorCopia();

                    int[] v1_xy = math.projecao2d( v1, this );
                    int[] v2_xy = math.projecao2d( v2, this );

                    int x1 = v1_xy[0];
                    int y1 = v1_xy[1];
                    int x2 = v2_xy[0];
                    int y2 = v2_xy[1];

                    g.setColor( arestasCor );
                    g.drawLine( x1, y1, x2, y2 );
                }
            }
        }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if ( mouseX != -1 && forma != null ) {
            int w = super.getWidth();
            int h = super.getHeight();
            int dx = e.getX() - mouseX;
            int dy = e.getY() - mouseY;
            
            double yang = ( (double)dx / (double)w ) * 2 * Math.PI;
            double xang = ( (double)dy / (double)h ) * 2 * Math.PI;
            
            //transformador.rot( forma, xang, yang, 0 );
            
            char r = Character.toLowerCase( rotCH );
            switch ( r ) {
                case 'z':
                    transformador.rotZ( forma, -yang ); 
                    break;
                default:
                    transformador.rotX( forma, -xang );                     
                    transformador.rotY( forma, -yang );
            }            
            
            super.repaint();
        }        
        mouseX = e.getX();
        mouseY = e.getY();        
    }
    
    @Override
    public void mouseReleased(MouseEvent e) {
        mouseX = -1;
        mouseY = -1;
    }
    
    @Override
    public void keyPressed(KeyEvent e ) {
        rotCH = e.getKeyChar();
    }

    @Override
    public void keyTyped(KeyEvent e) {}

    @Override
    public void keyReleased(KeyEvent e) {}    

    @Override
    public void mouseMoved(MouseEvent e) {}

    @Override
    public void mouseClicked(MouseEvent e) {}

    @Override
    public void mousePressed(MouseEvent e) {}
    
    @Override
    public void mouseEntered(MouseEvent e) {}

    @Override
    public void mouseExited(MouseEvent e) {}
        
    public void setForma3D( Forma3D forma ) {
        this.forma = forma;
    }
    
    public Transformador getTransformador() {
        return transformador;
    }
    
    public static void main(String[] args) {
        Cubo3D cubo = new Cubo3D();
        cubo.constroi( 200 ); 
        
        Principal p = new Principal();
        p.setForma3D( cubo );
        
        p.getTransformador().rotX( cubo, -Math.PI / 5.0d ); 
        p.getTransformador().rotY( cubo, -Math.PI / 6.0d ); 
        
        JFrame f = new JFrame();
        f.setTitle( "Desenho de cubo 3D" ); 
        f.setContentPane( p );
        f.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        f.setSize( 640, 480 );
        f.setLocationRelativeTo( f );
        f.setVisible( true );
        
        p.repaint();
        
    }
    
}
